/******************************************************************
SQL Saturday #110: March 10th, 2012, Tampa, FL

Dmitri Korotkevitch: "Service Broker - Why, When and Where

email: dmitri@aboutsqlserver.com
blog: http://aboutsqlserver.com

Server 1: Database initialization script

All foreign keys constraints are omitted

Disclamer: Scripts are provided for the demonstration purposes only.
Neither architecture nor database schema nor code are in production quality.
At bare minimum the following is missing:
	1. Exception & Poison Mesaging handling
	2. "Fire and Forget" pattern redesign	
******************************************************************/

use SqlSat110 
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		s.name = 'dbo' and t.name = 'SBLog'
)
	drop table dbo.SBLog
go


-- Let's create table for logging
create table dbo.SBLog
(
	ID int not null identity(1,1),
	ATime datetime not null
		constraint DEF_SBLog
		default getDate(),
	Queue sysname not null,
	MsgType sysname not null,
	MsgBody xml null,
	Comment varchar(max),
	
	constraint PK_SBLog
	primary key clustered(ID)
)
go


if exists
(
	select * 
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		s.name = 'dbo' and t.name = 'Customers'
)
	drop table dbo.Customers
go

create table dbo.Customers
(
	CustomerId int not null,
	CustomerName nvarchar(64) not null,
	NotificationAddress nvarchar(128) null,
	NotificationMethod tinyint not null
		constraint DEF_Customers_NotificationMethod
		default 0,
		
	constraint PK_Customers
	primary key clustered(CustomerId)
)
go

;with Ids(ID)
as
(
	select 1 
	union all 
	select ID + 1
	from Ids 
	where ID < 500000
)
insert into dbo.Customers(CustomerId,CustomerName,NotificationAddress,NotificationMethod)
	select 
		ID, 
		'Customer: ' + CONVERT(nvarchar(10),Id),
		'Address: ' + CONVERT(nvarchar(10),Id),
		ID % 2
	from Ids
option (maxrecursion 0)
go


if exists
(
	select * 
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		s.name = 'dbo' and t.name = 'Articles'
)
	drop table dbo.Articles
go

create table dbo.Articles
(
	ArticleId int not null,
	ArticleName nvarchar(64) not null,
	RemainingStock int not null
		constraint DEF_Articles_RemainingStock
		default 0,
		
	constraint PK_Articles
	primary key clustered(ArticleId)
)
go

;with Ids(ID)
as
(
	select 1 
	union all 
	select ID + 1
	from Ids 
	where ID < 250000
)
insert into dbo.Articles(ArticleId, ArticleName, RemainingStock)
	select 
		ID, 
		'Article: ' + CONVERT(nvarchar(10),Id),
		0
	from Ids
option (maxrecursion 0)
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		s.name = 'dbo' and t.name = 'ArticleInStockNotifications'
)
	drop table dbo.ArticleInStockNotifications
go

create table dbo.ArticleInStockNotifications
(
	ArticleId int not null,
	CustomerId int not null,
	
	constraint PK_ArticleInStockNotifications
	primary key clustered(ArticleId, CustomerId)
	
	-- foreign keys are omitted
)
go

;with CustIds(CID)
as
(
	select 1
	union all 
	select CID + 50000
	from CustIds 
	where CID < 500000
)
,ArIds(AID)
as
(
	select 1 
	union all 
	select AID + 1
	from ArIds 
	where AID < 250000
)
insert into dbo.ArticleInStockNotifications(ArticleId,CustomerId)
	select AID, CID
	from ArIds cross join CustIds
option (maxrecursion 0)
go

;with CustIds(CID)
as
(
	select 2
	union all 
	select CID + 50000
	from CustIds 
	where CID < 500000
)
,ArIds(AID)
as
(
	select 2 
	union all 
	select AID + 1
	from ArIds 
	where AID < 250000
)
insert into dbo.ArticleInStockNotifications(ArticleId,CustomerId)
	select AID, CID
	from ArIds cross join CustIds
option (maxrecursion 0)
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		s.name = 'dbo' and t.name = 'PendingInStockNotifications'
)
	drop table dbo.PendingInStockNotifications
go

create table dbo.PendingInStockNotifications
(
	ArticleId int not null,
	CustomerId int not null,
	
	constraint PK_PendingInStockNotifications
	primary key clustered(ArticleId, CustomerId)
)
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		s.name = 'dbo' and t.name = 'Orders'
)
	drop table dbo.Orders
go

create table dbo.Orders
(
	OrderId int not null identity(1,1),
	OrderDate smalldatetime not null
		constraint DEF_Orders
		default GetUTCDate(),
	CustomerId int not null,
	Amount money not null,
	Paid bit not null
		constraint DEF_Orders_Paid 
		default 0,
	ArticlesReserved bit not null
		constraint DEF_Orders_ArticlesReserved 
		default 0,
	Shipped bit not null
		constraint DEF_Orders_Shipped 
		default 0,
	ShipmentDate smalldatetime null,
	Active bit not null
		constraint DEF_Orders_Active 
		default 1,
	
	constraint PK_Orders
	primary key clustered(OrderId)
	
	-- foreign keys are omitted
)
go

if exists
(
	select * 
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		s.name = 'dbo' and t.name = 'OrderItems'
)
	drop table dbo.OrderItems
go

create table dbo.OrderItems
(
	OrderId int not null,
	ArticleId int not null,
	Quantity int not null,
	Price money not null,
	
	constraint PK_OrderItems
	primary key clustered(OrderId,ArticleId)
	
	-- foreign keys are omitted
)
go
