/******************************************************************
SQL Saturday #110: March 10th, 2012, Tampa, FL

Dmitri Korotkevitch: "Service Broker - Why, When and Where

email: dmitri@aboutsqlserver.com
blog: http://aboutsqlserver.com

Server 1: "What Time Is it". Demonstration of
Service Broker concepts

Disclamer: Scripts are provided for the demonstration purposes only.
Neither architecture nor database schema nor code are in production quality.
At bare minimum the following is missing:
	1. Exception & Poison Mesaging handling
	2. "Fire and Forget" pattern redesign
******************************************************************/

use SqlSat110
go

-- Let's enable Service Broker
alter database SqlSat110 set SINGLE_USER with rollback immediate
go

alter database SqlSat110 set ENABLE_BROKER
go

alter database SqlSat110 set MULTI_USER
go

/*** Hello World within 1 database*** */
/* Cleaning up objects */

if exists
(
	select * 
	from sys.procedures p join sys.schemas s on
		p.schema_id = s.schema_id
	where
		s.name = 'dbo' and p.name = 'WhatTimeTargetProc'
)
	drop proc dbo.WhatTimeTargetProc
go

if exists (
	select * 
	from sys.services 
	where name = 'WhatTimeInitiatorService' 
)
	drop service WhatTimeInitiatorService
go

if exists (
	select * 
	from sys.services 
	where name = 'WhatTimeTargetService' 
)
	drop service WhatTimeTargetService
go

if exists (
	select * 
	from sys.service_queues q join sys.schemas s on q.schema_id = s.schema_id
	where q.name = 'WhatTimeTargetQueue' and s.name = 'dbo'
)
	drop queue dbo.WhatTimeTargetQueue
go	

if exists (
	select * 
	from sys.service_queues q join sys.schemas s on q.schema_id = s.schema_id
	where q.name = 'WhatTimeInitiatorQueue' and s.name = 'dbo'
)
	drop queue dbo.WhatTimeInitiatorQueue
go	

if exists (
	select * 
	from sys.service_contracts 
	where name = 'http://aboutsqlserver.com/WhatTimeContract'
)
	drop contract [http://aboutsqlserver.com/WhatTimeContract]
go

if exists (
	select * 
	from sys.service_message_types 
	where name = 'http://aboutsqlserver.com/WhatTimeInitiatorMessage'
)
	drop message type [http://aboutsqlserver.com/WhatTimeInitiatorMessage]
go


if exists (
	select * 
	from sys.service_message_types 
	where name = 'http://aboutsqlserver.com/WhatTimeTargetMessage'
)
	drop message type [http://aboutsqlserver.com/WhatTimeTargetMessage]
go


/* Creating SB objects */

-- YELLOW PAPER
create message type [http://aboutsqlserver.com/WhatTimeInitiatorMessage]
authorization dbo
validation = WELL_FORMED_XML -- Plain English
go

-- GREEN PAPER
create message type [http://aboutsqlserver.com/WhatTimeTargetMessage]
authorization dbo
validation = WELL_FORMED_XML -- Plain English
go

-- CONTRACT Dmitri and XXXX signed
create contract [http://aboutsqlserver.com/WhatTimeContract]
authorization dbo
(
	[http://aboutsqlserver.com/WhatTimeInitiatorMessage] SENT BY INITIATOR, -- Dmitri writes on Yellow Paper
	[http://aboutsqlserver.com/WhatTimeTargetMessage] SENT BY TARGET -- XXXX writes on Green Paper
)
go

-- Box on Dmitri's desk
create  queue dbo.WhatTimeInitiatorQueue
with
	Status = ON,
	Retention = OFF
go

-- Box on XXXX's desk
create  queue dbo.WhatTimeTargetQueue
with
	Status = ON,
	Retention = OFF
go

-- Dmitri as the Service
create service WhatTimeInitiatorService
authorization dbo
on queue dbo.WhatTimeInitiatorQueue
([http://aboutsqlserver.com/WhatTimeContract])
go

-- XXXX as the Service
create service WhatTimeTargetService
authorization dbo
on queue dbo.WhatTimeTargetQueue
([http://aboutsqlserver.com/WhatTimeContract])
go


/* MANUAL RUN */
-- Sending Message
declare -- conversation handle
	@ch uniqueidentifier

-- We do it in transaction. Service Broker 
-- is fully transactional
begin tran	
	-- Start conversation
	begin dialog @ch
	from service WhatTimeInitiatorService
	to service N'WhatTimeTargetService'
	on contract [http://aboutsqlserver.com/WhatTimeContract]
	with encryption = off;
	
	-- Send Message on conversation
	send on conversation @ch
	message type [http://aboutsqlserver.com/WhatTimeInitiatorMessage]
	('<Msg>What Time Is it?</Msg>');
commit
go

-- end conversation 'C6BC078C-7865-E111-8718-00155D010B13' with cleanup

-- Let's check what is in the queues and conversations
select * from dbo.WhatTimeInitiatorQueue
select * from dbo.WhatTimeTargetQueue
go

-- Receiving message
declare
	@ch uniqueidentifier
	,@MessageBody as varbinary(max)
	,@MessageType as sysname
	,@Msg xml
	
begin tran
	-- Receiving 1 message from the Target Queue
	-- Waiting 10 seconds Max
	WAITFOR
	(
		RECEIVE top (1)
			@MessageType = message_type_name,
			@ch = conversation_handle,
			@MessageBody = message_body
		FROM dbo.WhatTimeTargetQueue
	), TIMEOUT 10000
	
	-- if there are no messages - exitng
	if @@ROWCOUNT = 0
	begin
		rollback
	end
	else begin
		select @MessageType as [MessageType]
		
		if @MessageType = 'http://aboutsqlserver.com/WhatTimeInitiatorMessage'
		begin
			select convert(xml,@MessageBody) as [Message];
		
			select @Msg = CONVERT(xml,
				N'<Msg>' + 
				convert(nvarchar(32), GETDATE()) + 
				N'</Msg>'
			);

			-- Send reply back		
			send on conversation @ch
			message type [http://aboutsqlserver.com/WhatTimeTargetMessage]
			(@Msg);
			
			-- Finishing conversation
			end conversation @ch
		end
	end
	-- other types including end conversation message here 
commit
go

-- Let's check what is in the queues and conversations
select * from dbo.WhatTimeInitiatorQueue
select * from dbo.WhatTimeTargetQueue
go

-- Receiving message
declare
	@ch uniqueidentifier
	,@MessageBody as varbinary(max)
	,@MessageType as sysname

while 1 = 1
begin
	begin tran
		-- Receiving 1 message from the Initiator Queue
		-- Waiting 1 second 
		WAITFOR
		(
			RECEIVE top (1)
				@MessageType = message_type_name,
				@ch = conversation_handle,
				@MessageBody = message_body
			FROM dbo.WhatTimeInitiatorQueue
		), TIMEOUT 1000

		if @@ROWCOUNT = 0
		begin
			rollback
			break
		end
		
		select @MessageType as [MessageType]
	
		if @MessageType = 'http://aboutsqlserver.com/WhatTimeTargetMessage'
		begin
			select convert(xml,@MessageBody) as [Message];
		end
		else if @MessageType = 'http://schemas.microsoft.com/SQL/ServiceBroker/EndDialog'
		begin -- End dialog is predefined type.
			end conversation @ch
		end
	commit
end
go

select * from dbo.WhatTimeInitiatorQueue
select * from dbo.WhatTimeTargetQueue
go


create proc dbo.WhatTimeTargetProc
with execute as owner
as
begin
	/* This SP created for demo of Activation only
	Don't use such SP in production!
	At bare minimum you want to:
	1. Add try/catch block with the logic for exception handling
	2. Check for message type in the logic
	3. Perhaps some code for poison messaging notification 
		(in case if you don't monitor queue status) */
	
	declare
		@ch uniqueidentifier
		,@MessageBody as varbinary(max)
		,@MessageType as sysname

	while 1 = 1
	begin
		begin tran
			WAITFOR
			(
				RECEIVE top (1)
					@MessageType = message_type_name,
					@ch = conversation_handle,
					@MessageBody = message_body
				FROM dbo.WhatTimeTargetQueue
			), TIMEOUT 10000

			if @@ROWCOUNT = 0
			begin
				rollback
				break
			end

			insert into dbo.SBLog(Queue, MsgType, MsgBody)			
			values('WhatTimeTargetQueue',@MessageType,CONVERT(xml,@MessageBody))
			
			end conversation @ch
		commit
	end
end
go

alter queue dbo.WhatTimeTargetQueue
with
	Status = ON,
	Retention = OFF,
	Activation
	(
		Status = ON,
		Procedure_Name = dbo.WhatTimeTargetProc,
		MAX_QUEUE_READERS = 1, 
		EXECUTE AS OWNER
	)
go


delete from dbo.SBLog
go

-- Demo of activation
declare
	@ch uniqueidentifier

begin tran	
	begin dialog @ch
	from service WhatTimeInitiatorService
	to service N'WhatTimeTargetService'
	on contract [http://aboutsqlserver.com/WhatTimeContract]
	with encryption = off;

	send on conversation @ch
	message type [http://aboutsqlserver.com/WhatTimeInitiatorMessage]
	('<Msg>Activation Demo</Msg>');

	-- Do not use 'Fire and Forget pattern' in production	
	end conversation @ch 
commit
go

select * from dbo.SBLog
go

select * from dbo.WhatTimeInitiatorQueue
select * from dbo.WhatTimeTargetQueue
go