/******************************************************************
SQL Saturday #110: March 10th, 2012, Tampa, FL

Dmitri Korotkevitch: "Service Broker - Why, When and Where

email: dmitri@aboutsqlserver.com
blog: http://aboutsqlserver.com

Server 1: Asyncronous Processing

Disclamer: Scripts are provided for the demonstration purposes only.
Neither architecture nor database schema nor code are in production quality.
At bare minimum the following is missing:
	1. Exception & Poison Mesaging handling
	2. "Fire and Forget" pattern redesign
******************************************************************/


use SQLSat110
go

if exists (
	select * 
	from sys.services 
	where name = 'InstockNotificationInitiatorService' 
)
	drop service InstockNotificationInitiatorService
go

if exists (
	select * 
	from sys.services 
	where name = 'InstockNotificationTargetService' 
)
	drop service InstockNotificationTargetService
go

if exists (
	select * 
	from sys.service_queues q join sys.schemas s on q.schema_id = s.schema_id
	where q.name = 'InstockNotificationTargetQueue' and s.name = 'dbo'
)
	drop queue dbo.InstockNotificationTargetQueue
go	

if exists (
	select * 
	from sys.service_queues q join sys.schemas s on q.schema_id = s.schema_id
	where q.name = 'InstockNotificationInitiatorQueue' and s.name = 'dbo'
)
	drop queue dbo.InstockNotificationInitiatorQueue
go	

if exists (
	select * 
	from sys.service_contracts 
	where name = 'http://aboutsqlserver.com/InstockNotificationContract'
)
	drop contract [http://aboutsqlserver.com/InstockNotificationContract]
go

if exists (
	select * 
	from sys.service_message_types 
	where name = 'http://aboutsqlserver.com/InstockNotificationMessage'
)
	drop message type [http://aboutsqlserver.com/InstockNotificationMessage]
go


if exists
(
	select * 
	from sys.procedures p join sys.schemas s on
		p.schema_id = s.schema_id
	where
		s.name = 'dbo' and p.name = 'InstockNotificationActivationProc'
)
	drop proc dbo.InstockNotificationActivationProc
go


if exists
(
	select * 
	from sys.procedures p join sys.schemas s on
		p.schema_id = s.schema_id
	where
		s.name = 'dbo' and p.name = 'ProcessInStockNotifications'
)
	drop proc dbo.ProcessInStockNotifications
go

if exists
(
	select * 
	from sys.procedures p join sys.schemas s on
		p.schema_id = s.schema_id
	where
		s.name = 'dbo' and p.name = 'ReceiveArticles'
)
	drop proc dbo.ReceiveArticles
go


if exists
(
	select * 
	from sys.procedures p join sys.schemas s on
		p.schema_id = s.schema_id
	where
		s.name = 'dbo' and p.name = 'ReceiveArticles2'
)
	drop proc dbo.ReceiveArticles2
go

if exists
(
	select * 
	from sys.types t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		s.name = 'dbo' and t.name = 'tvpIDList'
)
	drop type dbo.tvpIDList
go

if exists
(
	select * 
	from sys.types t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		s.name = 'dbo' and t.name = 'tvpArticles'
)
	drop type dbo.tvpArticles
go


create type dbo.tvpIDList as table
( ID int not null primary key)
go


create type dbo.tvpArticles as table
( 
	ArticleId int not null,
	ArticleName nvarchar(64) null,
	Quantity int not null,
	
	primary key(ArticleId)
)
go

create proc dbo.ProcessInStockNotifications
(
	@ArticleList dbo.tvpIDList READONLY
)
as
begin
	-- Let's assume we want to send email to all Customers with NotificationMethod = 0 
	-- and put info for all customers with NotificationMethod = 1 to PendingInStockNotifications

	declare
		@EMailNotification table
		(
			ArticleId int not null,
			ArticleName nvarchar(64) not null,
			EMailList nvarchar(max) not null,
			
			primary key(ArticleId)
		)
	
	declare
		@RC int
		
	-- Build the list of email addresses with Article information
	insert into @EMailNotification(ArticleId, ArticleName, EMailList)
		select 
			a.ArticleId, a.ArticleName, nt.EMails
		from 
			@ArticleList al join dbo.Articles a on
				al.ID = a.ArticleId
			cross apply
			(
				select	
				  CONVERT(nvarchar(max),
				  (
					select 
						c.NotificationAddress as [text()]
						,';' as [text()]
					from 
						dbo.ArticleInStockNotifications asn join dbo.Customers c on
							c.CustomerId = asn.CustomerId and 
							c.NotificationMethod = 0
					where
						a.ArticleId = asn.ArticleId						
					for xml path('')
				)) as EMails
			) nt
		where
			nt.EMails is not null

	select @RC = @@ROWCOUNT
	
	select 'Demo > @EmailNotification', * from 	@EMailNotification
	
	-- assuming we need to have 25 millisecond per email/article (select, generate and sent)
	declare
		@I int = 1

	while @I <= @RC
	begin
		waitfor delay '00:00:00.025'
		select @I += 1
	end
				
	-- Put remaining articles to another table
	insert into dbo.PendingInStockNotifications(ArticleId, CustomerId)
		select 
			a.ArticleId, asn.CustomerId
		from 
			@ArticleList al join dbo.Articles a on
				al.ID = a.ArticleId
			join dbo.ArticleInStockNotifications asn on
				a.ArticleId = asn.ArticleId
			join dbo.Customers c on
				c.CustomerId = asn.CustomerId and 
				c.NotificationMethod = 1
end
go


create proc dbo.ReceiveArticles
(
	@Articles dbo.tvpArticles READONLY
)
as
begin
	-- Receive articles

	declare
		@ArticleIds dbo.tvpIDList
		
	begin tran
		-- Statement below inserts/updates Articles to the table
		-- And add all articles that had Stock = 0 (subject for possible in-stock
		-- notifications) to @AtricleIds
		insert into @ArticleIds(ID)
			select ArticleId
			from 
				(
					merge into dbo.Articles as Target
					using (select * from @Articles) as Source
					on Source.ArticleId = Target.ArticleId
					when not matched then
						insert(ArticleId, ArticleName, RemainingStock)
						values(Source.ArticleId, Source.ArticleName, Source.Quantity)
					when matched then
						update set
							Target.RemainingStock = Target.RemainingStock + Source.Quantity
					output inserted.ArticleId, Deleted.RemainingStock
				) m(ArticleId, RemainingStock)
			where RemainingStock = 0
	commit			
	
	-- Call In Stock notification Proc
	if exists(select * from @ArticleIds)
		exec dbo.ProcessInStockNotifications @ArticleIds

end
go


-- Not let's test on the scope of 250 articles
update dbo.Articles set RemainingStock = 0 where ArticleId < 250
go

delete from dbo.PendingInStockNotifications
go


declare
	@ArticleList dbo.tvpArticles
	
;with Ids(ID)
as
(
	select 1 
	union all 
	select ID + 1
	from Ids 
	where ID < 250
)
insert into @ArticleList(ArticleId, ArticleName, Quantity)
	select ID, 'null', 1000 
	from Ids
option (maxrecursion 0)

exec dbo.ReceiveArticles @ArticleList
go




/* Creating SB objects */
create message type [http://aboutsqlserver.com/InstockNotificationMessage]
authorization dbo
validation = WELL_FORMED_XML
go

create contract [http://aboutsqlserver.com/InstockNotificationContract]
authorization dbo
(
	[http://aboutsqlserver.com/InstockNotificationMessage] SENT BY INITIATOR
)
go

create  queue dbo.InstockNotificationInitiatorQueue
with
	Status = ON,
	Retention = OFF
go

create proc dbo.InstockNotificationActivationProc
with execute as owner
as
	return
go

create  queue dbo.InstockNotificationTargetQueue
with
	Status = ON,
	Retention = OFF,
	Activation
	(
		Status = ON,
		Procedure_Name = dbo.InstockNotificationActivationProc,
		MAX_QUEUE_READERS = 1, 
		EXECUTE AS OWNER
	)
go


create service InstockNotificationInitiatorService
authorization dbo
on queue dbo.InstockNotificationInitiatorQueue
([http://aboutsqlserver.com/InstockNotificationContract])
go

create service InstockNotificationTargetService
authorization dbo
on queue dbo.InstockNotificationTargetQueue
([http://aboutsqlserver.com/InstockNotificationContract])
go

create proc dbo.ReceiveArticles2
(
	@Articles dbo.tvpArticles READONLY
)
as
begin
	declare
		@ArticleIds dbo.tvpIDList
		
	begin tran
		-- Same as before - update Articles and put
		-- ArticleIds for the articls with 0 stock 
		-- to @ArticleIds  table
		insert into @ArticleIds(ID)
			select ArticleId
			from 
				(
					merge into dbo.Articles as Target
					using (select * from @Articles) as Source
					on Source.ArticleId = Target.ArticleId
					when not matched then
						insert(ArticleId, ArticleName, RemainingStock)
						values(Source.ArticleId, Source.ArticleName, Source.Quantity)
					when matched then
						update set
							Target.RemainingStock = Target.RemainingStock + Source.Quantity
					output inserted.ArticleId, Deleted.RemainingStock
				) m(ArticleId, RemainingStock)
			where RemainingStock = 0

		if exists(select * from @ArticleIds)
		begin	
			-- But now let's use SB
			declare
				@IdsXml xml
				,@ch uniqueidentifier
		
			-- Serialize list of ArticleIds to XML
			select @IdsXml = 
				(
					select ID as "@ID"
					from @ArticleIds
					for xml path('Article'), root ('Articles')
				)

			begin dialog @ch
			from service InstockNotificationInitiatorService
			to service N'InstockNotificationTargetService'
			on contract [http://aboutsqlserver.com/InstockNotificationContract]
			with encryption = off;

			send on conversation @ch
			message type [http://aboutsqlserver.com/InstockNotificationMessage]
			(@IdsXml);
			
			-- Do not use "fire and forget" pattern in production
			end conversation @ch
		end
	commit
end
go


alter proc dbo.InstockNotificationActivationProc
with execute as owner
as
begin
	declare
		@ArticleIds dbo.tvpIDList
		
	declare
		@ch uniqueidentifier
		,@MessageBody as varbinary(max)
		,@MessageType as sysname
		,@MsgXml xml
		
	while 1 = 1
	begin
		begin tran
			WAITFOR
			(
				RECEIVE top (1)
					@MessageType = message_type_name,
					@ch = conversation_handle,
					@MessageBody = message_body
				FROM dbo.InstockNotificationTargetQueue
			), TIMEOUT 60000

			if @@ROWCOUNT = 0
			begin
				rollback
				break
			end

			insert into dbo.SBLog(Queue, MsgType, MsgBody)			
			values('InstockNotificationTargetQueue - start',@MessageType,CONVERT(xml,@MessageBody))
			
			if @MessageType = 'http://aboutsqlserver.com/InstockNotificationMessage'
			begin
				select @MsgXml = CONVERT(xml,@MessageBody)
				-- Deserialize the list of IDs
				insert into @ArticleIds(ID)
					select t.c.value('@ID','int')
					from @MsgXml.nodes('/Articles/Article') as t(c)
					
				if @@ROWCOUNT > 0
					exec dbo.ProcessInStockNotifications @ArticleIds
				
				end conversation @ch
			end
			-- else ... handle other messages types

			insert into dbo.SBLog(Queue, MsgType)			
			values('InstockNotificationTargetQueue - end',@MessageType)
		commit
	end
end
go



update dbo.Articles set RemainingStock = 0 where ArticleId < 250
go

delete from dbo.PendingInStockNotifications
go

delete from dbo.SBLog
go

declare
	@ArticleList dbo.tvpArticles
	
;with Ids(ID)
as
(
	select 1 
	union all 
	select ID + 1
	from Ids 
	where ID < 250
)
insert into @ArticleList(ArticleId, ArticleName, Quantity)
	select ID, 'null', 1000 
	from Ids
option (maxrecursion 0)

exec dbo.ReceiveArticles2 @ArticleList
go

select * from dbo.sbLog with (nolock)
go

