/******************************************************************
SQL Saturday #110: March 10th, 2012, Tampa, FL

Dmitri Korotkevitch: "Service Broker - Why, When and Where

email: dmitri@aboutsqlserver.com
blog: http://aboutsqlserver.com

Server 1: Setup Remote Service Broker Instance

Disclamer: Scripts are provided for the demonstration purposes only.
Neither architecture nor database schema nor code are in production quality.
At bare minimum the following is missing:
	1. Exception & Poison Mesaging handling
	2. "Fire and Forget" pattern redesign
******************************************************************/

/**********************************************
Step 1 - Create Service Broker Objects from 05. Ref Integrity SQL
**********************************************/



/**********************************************
Step 2 - Building Transport Security - 
a. change passwords below
b. change path to certificate file
**********************************************/
use master
go

create master key encryption 
by password = 'Pas$word1'
Go

create certificate SQL2008R2Str
with 
	SUBJECT = 'SQL2008R2Str.Endpoint',
    START_DATE = '01/01/2011',
    EXPIRY_DATE = '01/01/2099'
ACTIVE FOR BEGIN_DIALOG = ON;
GO

CREATE ENDPOINT ServiceBrokerEndPoint
STATE=STARTED
AS TCP (LISTENER_PORT = 4030)
FOR SERVICE_BROKER 
(
	AUTHENTICATION = CERTIFICATE SQL2008R2Str,
    ENCRYPTION = SUPPORTED
)
go

BACKUP CERTIFICATE SQL2008R2Str 
TO FILE = 'C:\Certs\SQL2008R2StrCertificate.cer';
GO

/*********************************************
 STOP HERE, Run the same steps on Server 2
 and transfer certificate files between the servers 
 *********************************************/
create certificate SqlDomainEndPointCertificate
from FILE = 'C:\Certs\SqlDomainEndPointCertificate.cer';
GO

CREATE LOGIN SqlDomainLogin
FROM CERTIFICATE SqlDomainEndPointCertificate;
GO

GRANT CONNECT ON ENDPOINT::ServiceBrokerEndPoint To SqlDomainLogin
GO

/**********************************************************
END STEP 2
**********************************************************/

/**********************************************
Step 3 - Building Dialog Level Security 
a. change passwords below
b. change path to certificate file
**********************************************/

use SQLSat110
go

create master key encryption 
by password = 'Pas$word1'
go

create certificate Server1UserCertificate
with 
	subject = 'Server1.DB.User',
    START_DATE = '01/01/2011',
    EXPIRY_DATE = '01/01/2099'
ACTIVE FOR BEGIN_DIALOG = ON;
GO

BACKUP CERTIFICATE Server1UserCertificate 
TO FILE='C:\Certs\Server1UserCertificate.cer';
GO

/*********************************************
 STOP HERE, Run the same steps on Server 2
 and transfer certificate files between the servers 
 *********************************************/
create user Server2User without login
GO

CREATE CERTIFICATE Server2UserCertificate
AUTHORIZATION Server2User
FROM FILE = 'C:\Certs\Server2UserCertificate.cer';
GO

GRANT CONNECT TO Server2User;
go

GRANT SEND ON SERVICE::RefIntegrityInitiatorService To Server2User;
GO

CREATE REMOTE SERVICE BINDING RefIntegrityServiceBinding
TO SERVICE 'RefIntegrityTargetService'
WITH USER = Server2User
go

/* END OF STEP 3 */

/**********************************************
Step 4 - Testing 
**********************************************/

-- Test 1 - Sending message from Server 1 to Server 2
declare 
	@ConversationHandle uniqueidentifier

begin tran
	begin try
		begin dialog @ConversationHandle
			from service RefIntegrityInitiatorService
			to service 'RefIntegrityTargetService'
			on contract [http://aboutsqlserver.com/RefIntegrityContract]
			with Encryption=off;

		select @ConversationHandle;
		
		send on conversation @ConversationHandle
		message type [http://aboutsqlserver.com/RefIntegrityMessage]
		(N'<test>test</test>')

		end conversation @ConversationHandle
		
		commit
	end try
	begin catch
		print 'error'
		rollback tran
	end catch 
go
select @@TRANCOUNT
go

-- Query those ones - msg should disappear from transmission queue after a few seconds. In this case check Archive that message has been delivered
select * from sys.transmission_queue
select * from sys.conversation_endpoints
select * from sys.routes
go

/* END OF STEP 4 */

