/******************************************************************
SQL Saturday #110: March 10th, 2012, Tampa, FL

Dmitri Korotkevitch: "Service Broker - Why, When and Where

email: dmitri@aboutsqlserver.com
blog: http://aboutsqlserver.com

Server 1: Setup Remote Service Broker Instance

Disclamer: Scripts are provided for the demonstration purposes only.
Neither architecture nor database schema nor code are in production quality.
At bare minimum the following is missing:
	1. Exception & Poison Mesaging handling
	2. "Fire and Forget" pattern redesign
******************************************************************/

/**********************************************
Step 1 - Create Service Broker Objects from 05. Ref Integrity SQL
**********************************************/


/**********************************************
Step 2 - Building Transport Security - 
a. change passwords below
b. change path to certificate file
**********************************************/

use master
go

create master key encryption 
by password = 'Pas$word1'
Go

create certificate SqlDomainEndPointCertificate
with 
	SUBJECT = 'SqlDomain.Endpoint',
    START_DATE = '01/01/2011',
    EXPIRY_DATE = '01/01/2099'
ACTIVE FOR BEGIN_DIALOG = ON;
GO

CREATE ENDPOINT ServiceBrokerEndPoint
STATE=STARTED
AS TCP (LISTENER_PORT = 4030)
FOR SERVICE_BROKER 
(
	AUTHENTICATION = CERTIFICATE SqlDomainEndPointCertificate,
    ENCRYPTION = SUPPORTED
)
go

BACKUP CERTIFICATE SqlDomainEndPointCertificate 
TO FILE = 'C:\Certs\SqlDomainEndPointCertificate.cer';
GO

/*********************************************
 STOP HERE, Run the same steps on prod (SB-Prod.sql) 
 and transfer certificate files between the servers 
 *********************************************/
create certificate SQL2008R2Str
from FILE = 'C:\Certs\SQL2008R2StrCertificate.cer';
GO

CREATE LOGIN Server1Login
FROM CERTIFICATE SQL2008R2Str;
GO

GRANT CONNECT ON ENDPOINT::ServiceBrokerEndPoint To Server1Login
GO

/**********************************************************
END STEP 2
**********************************************************/

/**********************************************
Step 3 - Building Dialog Level Security 
a. change passwords below
b. change path to certificate file
**********************************************/
use SQLSat110_2
go

create master key encryption 
by password = 'Pas$word1'
go

create certificate Server2UserCertificate
with 
	subject = 'Server2.DB.User',
    START_DATE = '01/01/2011',
    EXPIRY_DATE = '01/01/2099'
ACTIVE FOR BEGIN_DIALOG = ON;
GO

BACKUP CERTIFICATE Server2UserCertificate 
TO FILE='C:\Certs\Server2UserCertificate.cer';
GO


/*********************************************
 STOP HERE, Run the same steps on prod (SB-Prod.sql) 
 and transfer certificate files between the servers 
 *********************************************/
create user Server1User without login
GO

CREATE CERTIFICATE Server1UserCertificate
AUTHORIZATION Server1User
FROM FILE = 'C:\Certs\Server1UserCertificate.cer';
GO

GRANT CONNECT TO Server1User;
go

GRANT SEND ON SERVICE::RefIntegrityTargetService To Server1User;
GO

CREATE REMOTE SERVICE BINDING RefIntegrityServiceBinding
TO SERVICE 'RefIntegrityInitiatorService'
WITH USER = Server1User
go


/* END OF STEP 3 */

/**********************************************
Step 4 - Testing
**********************************************/

-- Test 1 - Sending message from Server1 to Server 2
-- If everything is fine - you should have 2 messages in the queue

select * from dbo.RefIntegrityTargetQueue
go


/*
select * from sys.endpoints
select * from sys.transmission_queue
select * from sys.conversation_endpoints
*/
-- end conversation '2604BAEB-E264-E111-960E-00155D010B08' with cleanup


/*****Receive the Message and end the conversation */
declare 
	@ConversationHandle as uniqueidentifier
	,@MessageBody as nvarchar(max)
	,@MessageType as sysname

begin tran
	begin try
		;RECEIVE top (1)
			@MessageType = message_type_name,
			@ConversationHandle = conversation_handle,
			@MessageBody = message_body
		FROM dbo.RefIntegrityTargetQueue;

		select @ConversationHandle, @MessageType, convert(xml,@MessageBody)
		END Conversation @ConversationHandle
		commit
	end try
	begin catch
		print 'error'
		select ERROR_MESSAGE()
		rollback
	end catch
go

