/******************************************************************
SQL Saturday #110: March 10th, 2012, Tampa, FL

Dmitri Korotkevitch: "Service Broker - Why, When and Where

email: dmitri@aboutsqlserver.com
blog: http://aboutsqlserver.com

Server 2: Cross Server Referential Integrity

Disclamer: Scripts are provided for the demonstration purposes only.
Neither architecture nor database schema nor code are in production quality.
At bare minimum the following is missing:
	1. Exception & Poison Mesaging handling
	2. "Fire and Forget" pattern redesign
******************************************************************/


use SQLSat110_2
go


if exists (
	select * 
	from sys.services 
	where name = 'RefIntegrityTargetService' 
)
	drop service RefIntegrityTargetService
go

if exists (
	select * 
	from sys.service_queues q join sys.schemas s on q.schema_id = s.schema_id
	where q.name = 'RefIntegrityTargetQueue' and s.name = 'dbo'
)
	drop queue dbo.RefIntegrityTargetQueue
go	

if exists (
	select * 
	from sys.service_contracts 
	where name = 'http://aboutsqlserver.com/RefIntegrityContract'
)
	drop contract [http://aboutsqlserver.com/RefIntegrityContract]
go

if exists (
	select * 
	from sys.service_message_types 
	where name = 'http://aboutsqlserver.com/RefIntegrityMessage'
)
	drop message type [http://aboutsqlserver.com/RefIntegrityMessage]
go

if exists
(
	select * 
	from sys.procedures p join sys.schemas s on
		p.schema_id = s.schema_id
	where
		s.name = 'dbo' and p.name = 'DeleteCustomer'
)
	drop proc dbo.DeleteCustomer
go

if exists 
(
	select * from sys.routes where name = 'RefIntegrity2to1Route'
)
	drop route RefIntegrity2to1Route
go

create route RefIntegrity2to1Route
with
	SERVICE_NAME = 'RefIntegrityInitiatorService',
	ADDRESS = 'TCP://11.2.1.100:4030'
GO


create message type [http://aboutsqlserver.com/RefIntegrityMessage]
authorization dbo
validation = WELL_FORMED_XML
go

create contract [http://aboutsqlserver.com/RefIntegrityContract]
authorization dbo
(
	[http://aboutsqlserver.com/RefIntegrityMessage] SENT BY INITIATOR
)
go

create  queue dbo.RefIntegrityTargetQueue
with
	Status = ON,
	Retention = OFF
go

create service RefIntegrityTargetService
authorization dbo
on queue dbo.RefIntegrityTargetQueue
([http://aboutsqlserver.com/RefIntegrityContract])
go




create proc dbo.DeleteCustomer
with execute as owner
as
begin
	set xact_abort on
	set nocount on

	declare 
		@ch as uniqueidentifier
		,@MessageBody as nvarchar(max)
		,@MessageType as sysname
		,@MsgXml xml

	while 1 = 1
	begin
		begin tran
			WAITFOR
			(
				RECEIVE top (1)
					@MessageType = message_type_name,
					@ch = conversation_handle,
					@MessageBody = message_body
				FROM dbo.RefIntegrityTargetQueue
			), TIMEOUT 60000

			if @@ROWCOUNT = 0
			begin
				rollback
				break
			end

			insert into dbo.SBLog(Queue, MsgType, MsgBody)			
			values('InstockNotificationTargetQueue',@MessageType,CONVERT(xml,@MessageBody))
			
			if @MessageType = 'http://aboutsqlserver.com/RefIntegrityMessage'
			begin
				select @MsgXml = CONVERT(xml,@MessageBody)
				delete from dbo.Customers
				where
					CustomerId in 
					(
						select t.c.value('@Id','int')
						from @MsgXml.nodes('/Deletion/Customer') as t(c)
					)

				end conversation @ch
			end
			-- else ... handle other messages types
		commit
	end
end
go

alter queue dbo.RefIntegrityTargetQueue
with
	Status = ON,
	Retention = OFF,
	Activation
	(
		Status = ON,
		Procedure_Name = dbo.DeleteCustomer,
		MAX_QUEUE_READERS = 1, 
		EXECUTE AS OWNER
	)
go

alter queue dbo.RefIntegrityTargetQueue
with
	Status = ON,
	Retention = OFF,
	Activation
	(
		Status = OFF,
		Procedure_Name = dbo.DeleteCustomer,
		MAX_QUEUE_READERS = 1, 
		EXECUTE AS OWNER
	)
go

declare
	@CustomerId int = 506
	
select * from dbo.Customers where CustomerId = @CustomerId
select * FROM dbo.RefIntegrityTargetQueue
select * from sys.transmission_queue
select * from sys.conversation_endpoints

delete from dbo.SBLog
go

select * from dbo.SBLog

