/******************************************************************
SQL Saturday #110: March 10th, 2012, Tampa, FL

Dmitri Korotkevitch: "Service Broker - Why, When and Where

email: dmitri@aboutsqlserver.com
blog: http://aboutsqlserver.com

Server 1: SOA - Moving Payment Service to remote location

Disclamer: Scripts are provided for the demonstration purposes only.
Neither architecture nor database schema nor code are in production quality.
At bare minimum the following is missing:
	1. Exception & Poison Mesaging handling
	2. "Fire and Forget" pattern redesign
******************************************************************/
use SQLSat110_2
go

if exists (
	select * 
	from sys.services 
	where name = 'SOAPaymentService' 
)
	drop service SOAPaymentService
go

if exists (
	select * 
	from sys.service_queues q join sys.schemas s on q.schema_id = s.schema_id
	where q.name = 'SOAPaymentQueue' and s.name = 'dbo'
)
	drop queue dbo.SOAPaymentQueue
go

if exists (
	select * 
	from sys.service_contracts 
	where name = 'http://aboutsqlserver.com/SOAPaymentContract'
)
	drop contract [http://aboutsqlserver.com/SOAPaymentContract]
go


if exists (
	select * 
	from sys.service_message_types 
	where name = 'http://aboutsqlserver.com/SOAPaymentRequest'
)
	drop message type [http://aboutsqlserver.com/SOAPaymentRequest]
go

if exists (
	select * 
	from sys.service_message_types 
	where name = 'http://aboutsqlserver.com/SOAPaymentResponse'
)
	drop message type [http://aboutsqlserver.com/SOAPaymentResponse]
go

if exists
(
	select * 
	from sys.procedures p join sys.schemas s on
		p.schema_id = s.schema_id
	where
		s.name = 'dbo' and p.name = 'SOAPaymentQueueProc'
)
	drop proc dbo.SOAPaymentQueueProc
go

if exists 
(
	select * from sys.routes where name = 'SOAPayment2to1Route'
)
	drop route SOAPayment2to1Route
go

if exists (
	select * 
	from sys.remote_service_bindings 
	where name = 'SOAServiceBinding'
)
	drop remote service binding SOAServiceBinding
go


create message type [http://aboutsqlserver.com/SOAPaymentRequest]
authorization dbo
validation = WELL_FORMED_XML
go

create message type [http://aboutsqlserver.com/SOAPaymentResponse]
authorization dbo
validation = WELL_FORMED_XML
go

create contract [http://aboutsqlserver.com/SOAPaymentContract]
authorization dbo
(
	[http://aboutsqlserver.com/SOAPaymentRequest] SENT BY INITIATOR,
	[http://aboutsqlserver.com/SOAPaymentResponse] SENT BY TARGET
	
)
go

create proc dbo.SOAPaymentQueueProc
with execute as owner
as
	return
go

create queue dbo.SOAPaymentQueue
with
	Status = ON,
	Retention = OFF,
	Activation
	(
		Status = ON,
		Procedure_Name = dbo.SOAPaymentQueueProc,
		MAX_QUEUE_READERS = 1, 
		EXECUTE AS OWNER
	)
go


create service SOAPaymentService
authorization dbo
on queue dbo.SOAPaymentQueue
(
	[http://aboutsqlserver.com/SOAPaymentContract]
)
go

create route SOAPayment2to1Route
with
	SERVICE_NAME = 'SOAOrderService',
	ADDRESS = 'TCP://11.2.1.100:4030'
GO


GRANT SEND ON SERVICE::SOAPaymentService To Server1User;
GO


CREATE REMOTE SERVICE BINDING SOAServiceBinding
TO SERVICE 'SOAOrderService'
WITH USER = Server1User
go


alter proc dbo.SOAPaymentQueueProc
with execute as owner
as
begin
	set nocount on
	
	declare
		@ch uniqueidentifier
		,@MessageBody as varbinary(max)
		,@MessageType as sysname
		,@MsgXml xml
		,@OrderId int
		,@CustomerId int
		,@PaymentInfo varchar(255)
		,@Action varchar(255)
		,@Response varchar(255)
		,@MSG2 xml
		
	while 1 = 1
	begin
		begin tran
			WAITFOR
			(
				RECEIVE top (1)
					@MessageType = message_type_name,
					@ch = conversation_handle,
					@MessageBody = message_body
				FROM dbo.SOAPaymentQueue
			), TIMEOUT 60000

			if @@ROWCOUNT = 0
			begin
				rollback
				break
			end

			insert into dbo.SBLog(Queue, MsgType, MsgBody)			
			values('SOAPaymentQueue',@MessageType,CONVERT(xml,@MessageBody))
			
			if @MessageType = 'http://aboutsqlserver.com/SOAPaymentRequest'
			begin
				select @MsgXml = CONVERT(xml,@MessageBody)
				select 
					@OrderId = @MsgXml.value('/Tran[1]/@OrderId','int')
					,@CustomerId = @MsgXml.value('/Tran[1]/@CustomerId','int')
					,@PaymentInfo = @MsgXml.value('/Tran[1]/@PaymentInfo','varchar(255)')
					,@Action = @MsgXml.value('/Tran[1]/@Action','varchar(255)')

				if @Action = 'AUTH'
				begin
					-- Authorizing payment
					select @Response = 'OK'
					waitfor delay '0:00:01.000'
				end
				else if @Action = 'REFUND'
				begin
					-- Authorizing payment
					select @Response = 'OK'
					waitfor delay '0:00:01.000'
				end
				
				select @Msg2 = 
					(
						select 
							@OrderId as [@OrderId],
							@Response as [@Response]
						for XML path('Tran')
					);
				
				send on conversation @ch
				message type [http://aboutsqlserver.com/SOAPaymentResponse]
				(@Msg2);		
			
				end conversation @ch
			end
			-- else ... handle other messages types
		commit
	end
end
go

delete from dbo.SBLog
go

select * from SBLog with (nolock)
